/************************************************************************************************************************
  TITLE:       The Iron Man Arm
  DESCRIPTION: This program uses flex sensor data to determine when to fire a BOOMco blaster using servos or light an LED
  AUTHOR:      David Cox
  MODIFIED:    5/4/2017
 ***********************************************************************************************************************/

#include <Servo.h> //Includes the servo library

#define flexFist 1 //Analog pin for the flex sensor signal from the back of the knuckle
#define flexWrist 2 //Analog pin for the flex sensor signal from the back of the wrist
#define servoL 9 //PWM pin for the left servo
#define servoR 10 //PWM pin for the right servo
#define ledPin 6 //Digital pin to the LEDs
#define resistorPin 12 //Digital pin for switching current through a resistor, to keep the battery from turning off

Servo myservoL; //Calls the servo library for the left and right servos, respectively
Servo myservoR;

int resistorOff = 10000; //These are the lengths of time in ms that the resistor at pin 12 will alternate off and on
int resistorOn = 500;
int resistorState = 0; //The resistor is initially off
int currentTime; //These variables are used to store the time values
int previousTime;

int flexDuration = 500; //Time range in ms over which flex sensor data is averaged for each reading
int fistTarget = 850; //Experimentally determined flex sensor signal at the desired knuckle bend to move the servos
int wristTarget = 835; //Experimentally determined flex sensor signal at the desired wrist bend to light the LEDs

int flagFist = 0; //These two variables indicate if the servos or the LEDs should be activated, respectively
int flagWrist = 0;

int downL = 178; //These four values are found by experiment
int upL = 112; //They are the servo positions in the open and closed states
int downR = 108;
int upR = 175;

int up = 1;
int down = 0;

int servoDelay = 10; //ms delay called after each 1 degree increment of servo motion
//higher value makes the servos move slower



void setup() {
  myservoL.attach(servoL); //The left and right servos are initialized and moved to the closed position
  myservoL.write(downL);
  myservoR.attach(servoR);
  myservoR.write(downR);

  pinMode(ledPin, OUTPUT); //The LED power pin is initialized and kept off
  analogWrite(ledPin, 0);
  pinMode(resistorPin, OUTPUT);
  digitalWrite(resistorPin, LOW);

  currentTime = millis();
  previousTime = currentTime;
}


void loop() {
  currentTime = millis(); //Stores the current program time value
  
  if (resistorState == 0 && ((currentTime - previousTime) > resistorOff)) { //Runs only if the resistor has been off long enough
    digitalWrite(resistorPin, HIGH); //Sends power to the resistor, to draw sufficient current from the power bank
    resistorState = 1;
    previousTime = currentTime;
  }

  if (resistorState == 1 && ((currentTime - previousTime) > resistorOn)) { //Runs only if the resistor has been on long enough
    digitalWrite(resistorPin, LOW); //Turns off the resistor, to conserve power
    resistorState = 0;
  }

  readFlex(flexFist, fistTarget); //Reading taken of flex sensor at knuckle; outputs whether reading is above target value
  readFlex(flexWrist, wristTarget); //Reading taken of flex sensor at wrist; outputs whether reading is above target value

  if (flagFist) { //Runs only if knuckle joint is sufficiently bent
    moveServos(up); //Servos are moved to the up position to fire the blaster
    delay(3000); //Wait time after firing
    moveServos(down); //Servos are returned to teh down position
  }

  if (flagWrist) { //Runs only if the wrist is bent sufficiently backward
    for (int u = 0; u < 100; u++){ //Loop for gradually increasing LED brightness using pulse-width modulation
      analogWrite(ledPin, u);
      delay(7);
    }
    digitalWrite(ledPin, HIGH); //Sets the LED to full brightness
    delay(250);
    for (int v = 200; v >0; v=v-5){ //Loop for gradually decreasing the LED brightness using pulse-width modulation
      analogWrite(ledPin, v);
      delay(5);
    }
    delay(10);
    digitalWrite(ledPin, LOW); //The LED is turned completely off
    delay(1000); //Delay to prevent the LED from coming back on too soon
  }
  
}
